@@ -57,18 +57,20 @@ struct srv_content_filtering_req_data {
     const srv_cf_profile_t *profile;
     /*the body data*/
     srv_cf_body_t body;
-    enum EncodeMethod enMethod;
+    int enMethod;
     int64_t maxBodyData;
     int64_t expectedData;
     /*flag for marking the eof*/
     int eof;
     int isText;
     int abort;
+    int isReqmod;
     srv_cf_results_t result;
 };
 
 static void generate_error_page(struct srv_content_filtering_req_data * data, ci_request_t * req, const char *tmpl);
 static void add_xheaders(struct srv_content_filtering_req_data * data, ci_request_t * req);
+static int encoding_method(const char *content_encoding);
 
 /*srv_content_filtering module text formating codes table*/
 static int fmt_srv_cf_action(ci_request_t *req, char *buf, int len, const char *param);
@@ -106,9 +108,9 @@ static struct ci_conf_entry srv_content_filtering_conf_variables[] = {
 CI_DECLARE_MOD_DATA ci_service_module_t service = {
     "srv_content_filtering",                         /* mod_name, The module name */
     "srv_content_filtering service",            /* mod_short_descr,  Module short description */
-    ICAP_RESPMOD,     /* mod_type, The service type is responce or request modification */
+    ICAP_RESPMOD|ICAP_REQMOD,     /* mod_type, The service type is responce or request modification */
     srv_content_filtering_init_service,              /* mod_init_service. Service initialization */
-    srv_content_filtering_post_init_service,         /* post_init_service. Service initialization after c-icap 
+    srv_content_filtering_post_init_service,         /* post_init_service. Service initialization after c-icap
 					configured. Not used here */
     srv_content_filtering_close_service,           /* mod_close_service. Called when service shutdowns. */
     srv_content_filtering_init_request_data,         /* mod_init_request_data */
@@ -126,7 +128,7 @@ int srv_content_filtering_init_service(ci_service_xdata_t * srv_xdata,
                       struct ci_server_conf *server_conf)
 {
      ci_debug_printf(5, "Initialization of srv_content_filtering module......\n");
-     
+
      /*Tell to the icap clients that we can support up to 1024 size of preview data*/
      ci_service_set_preview(srv_xdata, 1024);
 
@@ -145,14 +147,14 @@ int srv_content_filtering_post_init_service(ci_service_xdata_t * srv_xdata,
 }
 
 /* This function will be called when the service shutdown */
-void srv_content_filtering_close_service() 
+void srv_content_filtering_close_service()
 {
     srv_cf_filters_reset();
     srv_srv_cf_profiles_reset();
     ci_debug_printf(5,"Service shutdown!\n");
 }
 
-/*This function will be executed when a new request for srv_content_filtering service 
+/*This function will be executed when a new request for srv_content_filtering service
   arrives. This function will initialize the required structures and data
   to serve the request.
  */
@@ -167,14 +169,15 @@ void *srv_content_filtering_init_request_data(ci_request_t * req)
         return NULL;
     }
 
-    /*If the ICAP request encuspulates a HTTP objects which contains body data 
+    /*If the ICAP request encuspulates a HTTP objects which contains body data
       and not only headers allocate a ci_cached_file_t object to store the body data.
     */
     srv_cf_body_init(&srv_content_filtering_data->body);
     srv_content_filtering_data->eof = 0;
-    srv_content_filtering_data->enMethod = emNone;
+    srv_content_filtering_data->enMethod = CI_ENCODE_NONE;
     srv_content_filtering_data->isText = 0;
     srv_content_filtering_data->abort = 0;
+    srv_content_filtering_data->isReqmod = 0;
     srv_content_filtering_data->maxBodyData = 0;
     srv_content_filtering_data->expectedData = 0;
     srv_content_filtering_data->result.action = NULL;
@@ -192,7 +195,7 @@ void srv_content_filtering_release_request_data(void *data)
 {
     /*The data points to the srv_content_filtering_req_data struct we allocated in function srv_content_filtering_init_service */
     struct srv_content_filtering_req_data *srv_content_filtering_data = (struct srv_content_filtering_req_data *)data;
-    
+
     /*if we had body data, release the related allocated data*/
     srv_cf_body_free(&srv_content_filtering_data->body);
 
@@ -241,7 +244,7 @@ int srv_content_filtering_check_preview_handler(char *preview_data, int preview_
          return CI_MOD_ALLOW204;
      }
 
-     /*If there are is a Content-Length header, check it we do not want to 
+     /*If there are is a Content-Length header, check it we do not want to
       process body data with more than MaxBodyData size*/
      content_len = ci_http_content_length(req);
      ci_debug_printf(4, "Srv_Content_Filtering expected length: %"PRINTF_OFF_T"\n", (CAST_OFF_T) content_len);
@@ -259,20 +262,21 @@ int srv_content_filtering_check_preview_handler(char *preview_data, int preview_
      }
 
      if (srv_content_filtering_data->isText) {
-         
+
      }
 
      ci_debug_printf(8, "Srv_Content_Filtering service will process the request\n");
 
-     const char *contentEncoding = ci_http_response_get_header(req, "Content-Encoding");
-     if (!contentEncoding)
-         srv_content_filtering_data->enMethod = emNone;
-     else if (strstr(contentEncoding, "gzip") != NULL || strstr(contentEncoding, "deflate") != NULL)
-         srv_content_filtering_data->enMethod = emZlib;
-     else if (strstr(contentEncoding, "bzip2") != NULL)
-         srv_content_filtering_data->enMethod = emBzlib;
+     const char *contentEncoding = NULL;
+     if (req->type == ICAP_RESPMOD)
+         contentEncoding = ci_http_response_get_header(req, "Content-Encoding");
      else
-         srv_content_filtering_data->enMethod = emNone;
+         contentEncoding = ci_http_request_get_header(req, "Content-Encoding");
+
+     if (!contentEncoding)
+         srv_content_filtering_data->enMethod = CI_ENCODE_NONE;
+     else 
+         srv_content_filtering_data->enMethod = encoding_method(contentEncoding);
 
      srv_cf_body_build(&srv_content_filtering_data->body, content_len > 0 ? content_len + 1 : srv_content_filtering_data->maxBodyData);
 
@@ -286,12 +290,13 @@ int srv_content_filtering_check_preview_handler(char *preview_data, int preview_
          srv_content_filtering_data->eof = ci_req_hasalldata(req);
      }
 
+     srv_content_filtering_data->isReqmod = (req->type == ICAP_REQMOD ? 1 : 0);
      return CI_MOD_CONTINUE;
 }
 
-/* This function will called if we returned CI_MOD_CONTINUE in  
-   srv_content_filtering_check_preview_handler function, after we read all the data from 
-   the ICAP client 
+/* This function will called if we returned CI_MOD_CONTINUE in
+   srv_content_filtering_check_preview_handler function, after we read all the data from
+   the ICAP client
 */
 int srv_content_filtering_end_of_data_handler(ci_request_t * req)
 {
@@ -302,7 +307,7 @@ int srv_content_filtering_end_of_data_handler(ci_request_t * req)
 
     if (srv_content_filtering_data->abort) {
         /*We had already start sending data....*/
-        srv_content_filtering_data->eof = 1;        
+        srv_content_filtering_data->eof = 1;
         return CI_MOD_DONE;
     }
 
@@ -325,10 +330,17 @@ int srv_content_filtering_end_of_data_handler(ci_request_t * req)
 
     if (result->replaceBody && !ci_req_sent_data(req)) {
         srv_cf_body_replace_body(&srv_content_filtering_data->body, result->replaceBody);
-        ci_http_response_remove_header(req, "Content-Length");
         snprintf(tmpBuf, sizeof(tmpBuf), "Content-Length: %lld", (long long int)ci_membuf_size(result->replaceBody));
-        ci_http_response_add_header(req, tmpBuf);
-        result->replaceBody = NULL; /*Now srv_content_filtering_data->body points to this. 
+        if (srv_content_filtering_data->isReqmod) {
+            ci_http_request_remove_header(req, "Content-Encoding");
+            ci_http_request_remove_header(req, "Content-Length");
+            ci_http_request_add_header(req, tmpBuf);
+        } else {
+            ci_http_response_remove_header(req, "Content-Encoding");
+            ci_http_response_remove_header(req, "Content-Length");
+            ci_http_response_add_header(req, tmpBuf);
+        }
+        result->replaceBody = NULL; /*Now srv_content_filtering_data->body points to this.
                                      set it to NULL to avoid release twice later...*/
     }
 
@@ -361,7 +373,7 @@ int srv_content_filtering_end_of_data_handler(ci_request_t * req)
 }
 
 /* This function will called if we returned CI_MOD_CONTINUE in  srv_content_filtering_check_preview_handler
-   function, when new data arrived from the ICAP client and when the ICAP client is 
+   function, when new data arrived from the ICAP client and when the ICAP client is
    ready to get data.
 */
 int srv_content_filtering_io(char *wbuf, int *wlen, char *rbuf, int *rlen, int iseof,
@@ -373,7 +385,7 @@ int srv_content_filtering_io(char *wbuf, int *wlen, char *rbuf, int *rlen, int i
 
      /*write the data read from icap_client to the srv_content_filtering_data->body*/
      if(rlen && rbuf) {
-         if (srv_content_filtering_data->body.ring == NULL && 
+         if (srv_content_filtering_data->body.ring == NULL &&
              (srv_content_filtering_data->body.size + *rlen) > srv_content_filtering_data->maxBodyData) {
              ci_debug_printf(4, "Srv_Content_Filtering content-length:%" PRIu64 " bigger than maxBodyData:%" PRId64 "\n",
                              (srv_content_filtering_data->body.size + *rlen),
@@ -468,7 +480,7 @@ void add_xheaders(struct srv_content_filtering_req_data * data, ci_request_t * r
         ci_request_set_str_attribute(req, "srv_content_filtering:action_filter_score", buf);
 
         snprintf(buf, sizeof(buf), "X-Response-Desc: %s score=%d%c%d",
-                 data->result.action->matchingFilter->name, 
+                 data->result.action->matchingFilter->name,
                  data->result.action_score,
                  (data->result.action->scoreOperator == CF_OP_LESS ?  '<' :
                   (data->result.action->scoreOperator == CF_OP_GREATER ?  '>' : '=')),
@@ -478,9 +490,39 @@ void add_xheaders(struct srv_content_filtering_req_data * data, ci_request_t * r
     }
 }
 
+
+int encoding_method(const char *content_encoding)
+{
+#if defined(HAVE_CICAP_ENCODING_METHOD)
+    return ci_encoding_method(content_encoding);
+#else
+    if (!content_encoding)
+        return CI_ENCODE_NONE;
+
+    if (strcasestr(content_encoding, "gzip") != NULL) {
+        return CI_ENCODE_GZIP;
+    }
+
+    if (strcasestr(content_encoding, "deflate") != NULL) {
+        return CI_ENCODE_DEFLATE;
+    }
+#if defined(HAVE_CICAP_BROTLI)
+    if (strcasestr(content_encoding, "br") != NULL) {
+        return CI_ENCODE_BROTLI;
+    }
+#endif
+
+    if (strcasestr(content_encoding, "bzip2") != NULL) {
+        return CI_ENCODE_BZIP2;
+    }
+
+    return CI_ENCODE_UNKNOWN;
+#endif
+}
+
 int fmt_srv_cf_action(ci_request_t *req, char *buf, int len, const char *param)
 {
-    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);  
+    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);
     /*Do notwrite more than 512 bytes*/
     if (srv_cf_data && srv_cf_data->result.action)
         return snprintf(buf, len, "%s", srv_cf_action_str(srv_cf_data->result.action->action));
@@ -490,7 +532,7 @@ int fmt_srv_cf_action(ci_request_t *req, char *buf, int len, const char *param)
 
 int fmt_srv_cf_action_score(ci_request_t *req, char *buf, int len, const char *param)
 {
-    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);  
+    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);
     /*Do notwrite more than 512 bytes*/
     if (srv_cf_data  && srv_cf_data->result.action)
         return snprintf(buf, len, "%d", srv_cf_data->result.action_score);
@@ -500,10 +542,10 @@ int fmt_srv_cf_action_score(ci_request_t *req, char *buf, int len, const char *p
 
 int fmt_srv_cf_action_reason(ci_request_t *req, char *buf, int len, const char *param)
 {
-    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);  
+    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);
     /*Do notwrite more than 512 bytes*/
     if (srv_cf_data  && srv_cf_data->result.action)
-        return snprintf(buf, len, "%c%d", 
+        return snprintf(buf, len, "%c%d",
                         (srv_cf_data->result.action->scoreOperator == CF_OP_LESS ?  '<' :
                          (srv_cf_data->result.action->scoreOperator == CF_OP_GREATER ?  '>' : '=')),
                         srv_cf_data->result.action->score);
@@ -513,7 +555,7 @@ int fmt_srv_cf_action_reason(ci_request_t *req, char *buf, int len, const char *
 
 int fmt_srv_cf_scores_list(ci_request_t *req, char *buf, int len, const char *param)
 {
-    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);  
+    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);
     /*Do notwrite more than 512 bytes*/
     if (srv_cf_data  && srv_cf_data->result.scores)
         return srv_cf_print_scores_list(srv_cf_data->result.scores, buf, len);
@@ -523,7 +565,7 @@ int fmt_srv_cf_scores_list(ci_request_t *req, char *buf, int len, const char *pa
 
 int fmt_srv_cf_filter(ci_request_t *req, char *buf, int len, const char *param)
 {
-    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);  
+    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);
     /*Do notwrite more than 512 bytes*/
     if (srv_cf_data  && srv_cf_data->result.action)
         return snprintf(buf, len, "%s", srv_cf_data->result.action->matchingFilter->name);
@@ -533,7 +575,7 @@ int fmt_srv_cf_filter(ci_request_t *req, char *buf, int len, const char *param)
 
 int fmt_srv_cf_filter_matches(ci_request_t *req, char *buf, int len, const char *param)
 {
-    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);  
+    struct srv_content_filtering_req_data *srv_cf_data = ci_service_data(req);
     /*Do notwrite more than 512 bytes*/
     if (srv_cf_data  && srv_cf_data->result.action)
         return snprintf(buf, len, "%d", srv_cf_data->result.action_matchesCount);
